// ../protocol/dist/index.js
var EQSocketProtocolMessageType = /* @__PURE__ */ ((EQSocketProtocolMessageType2) => {
  EQSocketProtocolMessageType2[EQSocketProtocolMessageType2["DATA"] = 0] = "DATA";
  EQSocketProtocolMessageType2[EQSocketProtocolMessageType2["CONTROL"] = 1] = "CONTROL";
  EQSocketProtocolMessageType2[EQSocketProtocolMessageType2["ACK"] = 2] = "ACK";
  return EQSocketProtocolMessageType2;
})(EQSocketProtocolMessageType || {});
var EQSocketProtocolContentType = /* @__PURE__ */ ((EQSocketProtocolContentType2) => {
  EQSocketProtocolContentType2[EQSocketProtocolContentType2["UNDEFINED"] = 0] = "UNDEFINED";
  EQSocketProtocolContentType2[EQSocketProtocolContentType2["NULL"] = 1] = "NULL";
  EQSocketProtocolContentType2[EQSocketProtocolContentType2["BOOLEAN"] = 2] = "BOOLEAN";
  EQSocketProtocolContentType2[EQSocketProtocolContentType2["NUMBER"] = 3] = "NUMBER";
  EQSocketProtocolContentType2[EQSocketProtocolContentType2["STRING"] = 4] = "STRING";
  EQSocketProtocolContentType2[EQSocketProtocolContentType2["JSON"] = 5] = "JSON";
  EQSocketProtocolContentType2[EQSocketProtocolContentType2["BUFFER"] = 6] = "BUFFER";
  return EQSocketProtocolContentType2;
})(EQSocketProtocolContentType || {});
var EQSocketProtocolContentEncoding = /* @__PURE__ */ ((EQSocketProtocolContentEncoding2) => {
  EQSocketProtocolContentEncoding2[EQSocketProtocolContentEncoding2["RAW"] = 0] = "RAW";
  EQSocketProtocolContentEncoding2[EQSocketProtocolContentEncoding2["GZIP"] = 1] = "GZIP";
  EQSocketProtocolContentEncoding2[EQSocketProtocolContentEncoding2["DEFLATE"] = 2] = "DEFLATE";
  return EQSocketProtocolContentEncoding2;
})(EQSocketProtocolContentEncoding || {});
var QSocketProtocolEncodeError = class extends Error {
  /**
   * Creates an instance of QSocketProtocolEncodeError.
   *
   * @param message - A descriptive error message.
   * @param originalError - (Optional) The original error that caused this encoding error, if any.
   */
  constructor(message, originalError) {
    super(message);
    this.originalError = originalError;
    this.name = "QSocketProtocolEncodeError";
    if (originalError) {
      this.stack += `
Caused by: ${originalError.stack}`;
    }
  }
};
var QSocketProtocolDecodeError = class extends Error {
  /**
   * Creates an instance of QSocketProtocolDecodeError.
   *
   * @param message - A descriptive error message.
   * @param originalError - (Optional) The original error that caused this decoding error, if any.
   */
  constructor(message, originalError) {
    super(message);
    this.originalError = originalError;
    this.name = "QSocketProtocolDecodeError";
    if (originalError) {
      this.stack += `
Caused by: ${originalError.stack}`;
    }
  }
};
var hasBuffer = typeof Buffer !== "undefined" && typeof Buffer.isBuffer === "function";
function isBuffer(obj) {
  return hasBuffer && Buffer.isBuffer(obj);
}
var QSocketProtocol = class {
  /**
   * @description Initializes the `QSocketProtocol` instance with an optional compressor and a specified maximum size for uncompressed data.
   *
   * @param compressor - Optional object implementing `TQSocketProtocolCompressor` for data compression and decompression.
   * @param maxUncompressedSize - Maximum size in bytes for uncompressed data. Default is 10KB.
   */
  constructor(compressor, maxUncompressedSize = 10 * 1024) {
    this.compressor = compressor;
    this.maxUncompressedSize = maxUncompressedSize;
  }
  /**
   * @description Encodes a message into a `Buffer` or `Uint8Array`, applying optional compression if the message exceeds the maximum uncompressed size.
   *
   * @param message - A structured protocol message to encode.
   * @returns A `Promise` resolving to the encoded message as `Buffer` or `Uint8Array`, or an `EncodeError` on failure.
   */
  async to(message) {
    try {
      const chunkBuffers = [];
      for (const chunk of message) {
        const chunkBuffer = await this.encodeChunk(chunk);
        const chunkLengthBuffer = this.writeUInt32(chunkBuffer.length);
        const chunkEntryBuffer = this.concatBuffers([chunkLengthBuffer, chunkBuffer]);
        chunkBuffers.push(chunkEntryBuffer);
      }
      const messageData = this.concatBuffers(chunkBuffers);
      let compressedMessageData = messageData;
      let compressionFlag = 0;
      let compressionFormat = 0;
      const uncompressedLength = messageData.length;
      let compressedLength = messageData.length;
      if (this.compressor && uncompressedLength > this.maxUncompressedSize) {
        compressedMessageData = await this.compressor.toGzip(messageData);
        compressedLength = compressedMessageData.length;
        compressionFlag = 1;
        compressionFormat = 1;
      }
      const compressionFlagByte = this.writeUInt8(compressionFlag);
      const compressionFormatByte = this.writeUInt8(compressionFormat);
      const uncompressedLengthBuffer = this.writeUInt32(uncompressedLength);
      const compressedLengthBuffer = this.writeUInt32(compressedLength);
      const headerBuffers = [compressionFlagByte, compressionFormatByte, uncompressedLengthBuffer, compressedLengthBuffer];
      const finalBuffers = [...headerBuffers, compressedMessageData];
      const finalMessageBuffer = this.concatBuffers(finalBuffers);
      return finalMessageBuffer;
    } catch (error) {
      return new QSocketProtocolEncodeError(error.message);
    }
  }
  /**
   * @description Decodes a message from a `Buffer` or `Uint8Array`, decompressing if necessary, and converts it into structured protocol chunks.
   *
   * @param buffer - Encoded message data.
   * @returns A `Promise` resolving to a structured protocol message, or a `DecodeError` on failure.
   */
  async from(buffer) {
    try {
      let offset = 0;
      if (buffer.length < 10) {
        throw new QSocketProtocolDecodeError("Buffer too small to contain header");
      }
      const compressionFlag = this.readUInt8(buffer, offset);
      offset += 1;
      const compressionFormat = this.readUInt8(buffer, offset);
      offset += 1;
      const uncompressedLength = this.readUInt32(buffer, offset);
      offset += 4;
      const compressedLength = this.readUInt32(buffer, offset);
      offset += 4;
      if (buffer.length < offset + compressedLength) {
        throw new QSocketProtocolDecodeError("Buffer too small for compressed data");
      }
      const compressedMessageData = buffer.subarray(offset, offset + compressedLength);
      offset += compressedLength;
      if (compressedMessageData.length !== compressedLength) {
        throw new QSocketProtocolDecodeError("Compressed data length mismatch");
      }
      let messageData = compressedMessageData;
      if (compressionFlag === 1) {
        if (!this.compressor) {
          throw new QSocketProtocolDecodeError("Compressor not available for decompression");
        }
        if (compressionFormat === 1) {
          messageData = await this.compressor.fromGzip(compressedMessageData);
        } else if (compressionFormat === 2) {
          messageData = await this.compressor.fromDeflate(compressedMessageData);
        } else {
          throw new QSocketProtocolDecodeError("Unknown compression format");
        }
        if (messageData.length !== uncompressedLength) {
          throw new QSocketProtocolDecodeError("Uncompressed data length mismatch after decompression");
        }
      } else {
        if (messageData.length !== uncompressedLength) {
          throw new QSocketProtocolDecodeError("Uncompressed data length mismatch");
        }
      }
      let messageOffset = 0;
      const message = [];
      while (messageOffset < messageData.length) {
        if (messageData.length - messageOffset < 4) {
          throw new QSocketProtocolDecodeError("Insufficient data for chunk length");
        }
        const chunkLength = this.readUInt32(messageData, messageOffset);
        messageOffset += 4;
        if (messageData.length - messageOffset < chunkLength) {
          throw new QSocketProtocolDecodeError("Insufficient data for chunk");
        }
        const chunkBuffer = messageData.subarray(messageOffset, messageOffset + chunkLength);
        messageOffset += chunkLength;
        const chunk = await this.decodeChunk(chunkBuffer);
        message.push(chunk);
      }
      if (messageOffset !== messageData.length) {
        throw new QSocketProtocolDecodeError("Extra data after message parsing");
      }
      return message;
    } catch (error) {
      return new QSocketProtocolDecodeError(error.message);
    }
  }
  /**
   * @description Encodes an individual message chunk, converting metadata and payload into `Buffer` or `Uint8Array` with optional compression.
   *
   * @param chunk - A protocol chunk with metadata and payload data.
   * @returns A `Promise` resolving to the encoded chunk as `Buffer` or `Uint8Array`.
   */
  async encodeChunk(chunk) {
    if (!chunk.meta || typeof chunk.meta !== "object") {
      throw new Error("Invalid chunk meta");
    }
    const metaString = JSON.stringify(chunk.meta);
    const metaBuffer = this.encodeString(metaString);
    const metaLengthBuffer = this.writeUInt32(metaBuffer.length);
    const dataBuffer = await this.encodePayloadData(chunk.payload.data, chunk.payload["Content-Type"]);
    let payloadBuffer = dataBuffer;
    if (chunk.payload["Content-Encoding"] === 1) {
      if (this.compressor) {
        payloadBuffer = await this.compressor.toGzip(dataBuffer);
      } else {
        throw new Error("Compressor not available for GZIP encoding");
      }
    } else if (chunk.payload["Content-Encoding"] === 2) {
      if (this.compressor) {
        payloadBuffer = await this.compressor.toDeflate(dataBuffer);
      } else {
        throw new Error("Compressor not available for DEFLATE encoding");
      }
    } else if (chunk.payload["Content-Encoding"] !== 0) {
      throw new Error("Unknown Content-Encoding");
    }
    const payloadLengthBuffer = this.writeUInt32(payloadBuffer.length);
    const contentTypeByte = this.writeUInt8(chunk.payload["Content-Type"]);
    const contentEncodingByte = this.writeUInt8(chunk.payload["Content-Encoding"]);
    const buffers = [metaLengthBuffer, metaBuffer, payloadLengthBuffer, contentTypeByte, contentEncodingByte, payloadBuffer];
    const chunkBuffer = this.concatBuffers(buffers);
    return chunkBuffer;
  }
  /**
   * @description Decodes a chunk from `Buffer` or `Uint8Array` format back into a protocol chunk, decompressing if necessary.
   *
   * @param chunkBuffer - Encoded chunk data.
   * @returns A `Promise` resolving to a structured protocol chunk.
   */
  async decodeChunk(chunkBuffer) {
    let offset = 0;
    if (chunkBuffer.length < 4) {
      throw new Error("Chunk buffer too small for meta length");
    }
    const metaLength = this.readUInt32(chunkBuffer, offset);
    offset += 4;
    if (chunkBuffer.length < offset + metaLength) {
      throw new Error("Chunk buffer too small for metadata");
    }
    const metaBuffer = chunkBuffer.subarray(offset, offset + metaLength);
    offset += metaLength;
    const metaString = this.decodeString(metaBuffer);
    const meta = JSON.parse(metaString);
    if (chunkBuffer.length < offset + 4) {
      throw new Error("Chunk buffer too small for payload length");
    }
    const payloadLength = this.readUInt32(chunkBuffer, offset);
    offset += 4;
    if (chunkBuffer.length < offset + 2) {
      throw new Error("Chunk buffer too small for content type and encoding");
    }
    const contentType = this.readUInt8(chunkBuffer, offset);
    offset += 1;
    const contentEncoding = this.readUInt8(chunkBuffer, offset);
    offset += 1;
    if (chunkBuffer.length < offset + payloadLength) {
      throw new Error("Chunk buffer too small for payload data");
    }
    const payloadBuffer = chunkBuffer.subarray(offset, offset + payloadLength);
    offset += payloadLength;
    let dataBuffer = payloadBuffer;
    if (contentEncoding === 1) {
      if (!this.compressor) {
        throw new Error("Compressor not available for decompression");
      }
      dataBuffer = await this.compressor.fromGzip(payloadBuffer);
    } else if (contentEncoding === 2) {
      if (!this.compressor) {
        throw new Error("Compressor not available for decompression");
      }
      dataBuffer = await this.compressor.fromDeflate(payloadBuffer);
    } else if (contentEncoding !== 0) {
      throw new Error("Unknown Content-Encoding");
    }
    const data = this.decodePayloadData(dataBuffer, contentType);
    const chunk = {
      meta,
      payload: {
        data,
        "Content-Type": contentType,
        "Content-Encoding": contentEncoding
      }
    };
    return chunk;
  }
  /**
   * @description Encodes payload data according to its specified content type, converting it into a `Buffer` or `Uint8Array`.
   *
   * @param data - The payload data to encode.
   * @param contentType - Content type indicating the format of `data`.
   * @returns A `Promise` resolving to the encoded data as `Buffer` or `Uint8Array`.
   */
  async encodePayloadData(data, contentType) {
    switch (contentType) {
      case 0:
        return new Uint8Array(0);
      case 1:
        return new Uint8Array(0);
      case 2:
        return this.writeUInt8(data ? 1 : 0);
      case 3:
        return this.writeDouble(data);
      case 4:
        return this.encodeString(data);
      case 5:
        return this.encodeString(JSON.stringify(data));
      case 6:
        return isBuffer(data) ? data : new Uint8Array(data);
      default:
        throw new Error("Unknown content type");
    }
  }
  /**
   * @description Decodes payload data from a `Buffer` or `Uint8Array` based on its specified content type.
   *
   * @param buffer - Encoded data buffer.
   * @param contentType - Content type indicating the format of the buffer.
   * @returns The decoded data in its original format.
   */
  decodePayloadData(buffer, contentType) {
    switch (contentType) {
      case 0:
        return void 0;
      case 1:
        return null;
      case 2:
        const value = this.readUInt8(buffer, 0);
        return value !== 0;
      case 3:
        return this.readDouble(buffer, 0);
      case 4:
        return this.decodeString(buffer);
      case 5:
        return JSON.parse(this.decodeString(buffer));
      case 6:
        return buffer;
      default:
        throw new Error("Unknown content type");
    }
  }
  /**
   * @description Encodes a string into a UTF-8 `Buffer` or `Uint8Array`, depending on environment support.
   *
   * @param str - The string to encode.
   * @returns Encoded UTF-8 representation as `Buffer` or `Uint8Array`.
   */
  encodeString(str) {
    if (typeof TextEncoder !== "undefined") {
      const encoder = new TextEncoder();
      return encoder.encode(str);
    } else if (hasBuffer) {
      return Buffer.from(str, "utf8");
    } else {
      const utf8 = unescape(encodeURIComponent(str));
      const result = new Uint8Array(utf8.length);
      for (let i = 0; i < utf8.length; i++) {
        result[i] = utf8.charCodeAt(i);
      }
      return result;
    }
  }
  /**
   * @description Decodes a UTF-8 encoded `Buffer` or `Uint8Array` back into a string.
   *
   * @param buffer - UTF-8 encoded buffer.
   * @returns The decoded string.
   */
  decodeString(buffer) {
    if (typeof TextDecoder !== "undefined") {
      const decoder = new TextDecoder("utf8");
      return decoder.decode(buffer);
    } else if (hasBuffer && isBuffer(buffer)) {
      return buffer.toString("utf8");
    } else {
      let result = "";
      for (let i = 0; i < buffer.length; i++) {
        result += String.fromCharCode(buffer[i]);
      }
      return decodeURIComponent(escape(result));
    }
  }
  /**
   * @description Encodes a 32-bit unsigned integer into a `Buffer` or `Uint8Array` in big-endian format.
   *
   * @param value - The integer to encode.
   * @returns The encoded integer as `Buffer` or `Uint8Array`.
   */
  writeUInt32(value) {
    if (hasBuffer) {
      const buffer = Buffer.alloc(4);
      buffer.writeUInt32BE(value, 0);
      return buffer;
    } else {
      const buffer = new Uint8Array(4);
      const view = new DataView(buffer.buffer);
      view.setUint32(0, value, false);
      return buffer;
    }
  }
  /**
   * @description Reads a 32-bit unsigned integer from a `Buffer` or `Uint8Array` in big-endian format.
   *
   * @param buffer - The buffer containing the integer.
   * @param offset - The offset position within the buffer to start reading.
   * @returns The decoded integer.
   */
  readUInt32(buffer, offset) {
    if (hasBuffer && isBuffer(buffer)) {
      return buffer.readUInt32BE(offset);
    } else {
      const view = new DataView(buffer.buffer, buffer.byteOffset + offset, 4);
      return view.getUint32(0, false);
    }
  }
  /**
   * @description Writes an 8-bit unsigned integer into a `Buffer` or `Uint8Array`.
   *
   * @param value - The integer to write.
   * @returns The encoded integer as `Buffer` or `Uint8Array`.
   */
  writeUInt8(value) {
    const buffer = new Uint8Array(1);
    buffer[0] = value & 255;
    return buffer;
  }
  /**
   * @description Reads an 8-bit unsigned integer from a `Buffer` or `Uint8Array`.
   *
   * @param buffer - The buffer containing the integer.
   * @param offset - The offset position within the buffer to start reading.
   * @returns The decoded integer.
   */
  readUInt8(buffer, offset) {
    return buffer[offset];
  }
  /**
   * @description Encodes a double-precision floating-point number into a `Buffer` or `Uint8Array`.
   *
   * @param value - The floating-point number to encode.
   * @returns The encoded value as `Buffer` or `Uint8Array`.
   */
  writeDouble(value) {
    if (hasBuffer) {
      const buffer = Buffer.alloc(8);
      buffer.writeDoubleBE(value, 0);
      return buffer;
    } else {
      const buffer = new Uint8Array(8);
      const view = new DataView(buffer.buffer);
      view.setFloat64(0, value, false);
      return buffer;
    }
  }
  /**
   * @description Reads a double-precision floating-point number from a `Buffer` or `Uint8Array`.
   *
   * @param buffer - The buffer containing the double value.
   * @param offset - The offset position within the buffer to start reading.
   * @returns The decoded floating-point number.
   */
  readDouble(buffer, offset) {
    if (hasBuffer && isBuffer(buffer)) {
      return buffer.readDoubleBE(offset);
    } else {
      const view = new DataView(buffer.buffer, buffer.byteOffset + offset, 8);
      return view.getFloat64(0, false);
    }
  }
  /**
   * @description Concatenates multiple `Buffer` or `Uint8Array` instances into a single `Buffer` or `Uint8Array`.
   *
   * @param buffers - An array of buffers to concatenate.
   * @returns A single concatenated `Buffer` or `Uint8Array`.
   */
  concatBuffers(buffers) {
    if (hasBuffer) {
      return Buffer.concat(buffers.map((buf) => isBuffer(buf) ? buf : Buffer.from(buf)));
    } else {
      let totalLength = buffers.reduce((sum, buf) => sum + buf.length, 0);
      const result = new Uint8Array(totalLength);
      let offset = 0;
      for (const buf of buffers) {
        result.set(buf, offset);
        offset += buf.length;
      }
      return result;
    }
  }
};

// src/core/QSocketHelpers.ts
var contentTypeMap = /* @__PURE__ */ new Map([
  [EQSocketProtocolContentType.UNDEFINED, "undefined"],
  [EQSocketProtocolContentType.NULL, "null"],
  [EQSocketProtocolContentType.BOOLEAN, "boolean"],
  [EQSocketProtocolContentType.NUMBER, "number"],
  [EQSocketProtocolContentType.STRING, "string"],
  [EQSocketProtocolContentType.JSON, "json"],
  [EQSocketProtocolContentType.BUFFER, "buffer"]
]);
var reverseContentTypeMap = /* @__PURE__ */ new Map([
  ["undefined", EQSocketProtocolContentType.UNDEFINED],
  ["null", EQSocketProtocolContentType.NULL],
  ["boolean", EQSocketProtocolContentType.BOOLEAN],
  ["number", EQSocketProtocolContentType.NUMBER],
  ["string", EQSocketProtocolContentType.STRING],
  ["json", EQSocketProtocolContentType.JSON],
  ["buffer", EQSocketProtocolContentType.BUFFER]
]);
var contentEncodingMap = /* @__PURE__ */ new Map([
  [EQSocketProtocolContentEncoding.RAW, "raw"],
  [EQSocketProtocolContentEncoding.GZIP, "gzip"],
  [EQSocketProtocolContentEncoding.DEFLATE, "deflate"]
]);
var reverseContentEncodingMap = /* @__PURE__ */ new Map([
  ["raw", EQSocketProtocolContentEncoding.RAW],
  ["gzip", EQSocketProtocolContentEncoding.GZIP],
  ["deflate", EQSocketProtocolContentEncoding.DEFLATE]
]);
function determineContentType(data, contentType) {
  if (contentType) {
    const type = reverseContentTypeMap.get(contentType);
    if (type !== void 0) return type;
  }
  switch (typeof data) {
    case "undefined":
      return EQSocketProtocolContentType.UNDEFINED;
    case "boolean":
      return EQSocketProtocolContentType.BOOLEAN;
    case "number":
      return EQSocketProtocolContentType.NUMBER;
    case "string":
      return EQSocketProtocolContentType.STRING;
    case "symbol":
      return EQSocketProtocolContentType.UNDEFINED;
    case "object":
      if (data === null) return EQSocketProtocolContentType.NULL;
      if (Buffer.isBuffer(data)) return EQSocketProtocolContentType.BUFFER;
      return EQSocketProtocolContentType.JSON;
    default:
      return EQSocketProtocolContentType.UNDEFINED;
  }
}
function determineContentEncoding(contentEncoding) {
  if (contentEncoding) {
    const encoding = reverseContentEncodingMap.get(contentEncoding);
    if (encoding !== void 0) return encoding;
  }
  return EQSocketProtocolContentEncoding.RAW;
}
function getContentTypeString(contentType) {
  var _a;
  if (contentType === void 0) return "undefined";
  return (_a = contentTypeMap.get(contentType)) != null ? _a : "undefined";
}
function getContentEncodingString(contentEncoding) {
  var _a;
  if (contentEncoding === void 0) return "raw";
  return (_a = contentEncodingMap.get(contentEncoding)) != null ? _a : "raw";
}
function createErrorAckMessage(sourceChunk, errors) {
  return {
    meta: {
      type: EQSocketProtocolMessageType.ACK,
      uuid: sourceChunk.meta.uuid
    },
    payload: {
      data: errors.map((error) => error.message),
      "Content-Type": EQSocketProtocolContentType.JSON,
      "Content-Encoding": EQSocketProtocolContentEncoding.RAW
    }
  };
}
function createConfirmAckMessage(chunk, result) {
  return [
    {
      meta: {
        type: EQSocketProtocolMessageType.ACK,
        uuid: chunk.meta.uuid
      },
      payload: {
        data: result,
        "Content-Type": EQSocketProtocolContentType.BOOLEAN,
        "Content-Encoding": EQSocketProtocolContentEncoding.RAW
      }
    }
  ];
}
function createPingMessage(uuid) {
  return [
    {
      meta: {
        type: EQSocketProtocolMessageType.CONTROL,
        uuid
      },
      payload: {
        data: {
          command: "ping"
        },
        "Content-Type": EQSocketProtocolContentType.STRING,
        "Content-Encoding": EQSocketProtocolContentEncoding.RAW
      }
    }
  ];
}
function createPongMessage(uuid) {
  return [
    {
      meta: {
        type: EQSocketProtocolMessageType.ACK,
        uuid
      },
      payload: {
        data: "pong",
        "Content-Type": EQSocketProtocolContentType.STRING,
        "Content-Encoding": EQSocketProtocolContentEncoding.RAW
      }
    }
  ];
}

// src/core/QSocketEventEmetter.ts
var QSocketEventEmetterBase = class {
  constructor() {
    /**
     * Map of all event listeners, supporting multiple listeners for each event type.
     * @private
     * @type {Map<string, IQSocketListener<any, any>[]>}
     */
    this.listeners = /* @__PURE__ */ new Map();
    /**
     * Listeners for the "connection" event, triggered upon establishing a new connection.
     * @private
     * @type {((connection: QSocketConnection) => void)[]}
     */
    this.connectionListeners = [];
    /**
     * Listeners for the "disconnection" event, triggered when a connection is terminated.
     * @private
     * @type {(() => void)[]}
     */
    this.disconnectionListeners = [];
  }
  addEventListener(event, listener, type, contentType, contentEncoding) {
    let listeners = this.listeners.get(event);
    if (!listeners) {
      listeners = [];
      this.listeners.set(event, listeners);
    }
    listeners.push({
      type,
      listener,
      contentType,
      contentEncoding
    });
  }
  removeEventListener(event, listener) {
    const listeners = this.listeners.get(event);
    if (!listeners) return;
    const index = listeners.findIndex((item) => item.listener === listener);
    if (index !== -1) listeners.splice(index, 1);
  }
};
var QSocketConnectionEventEmitter = class extends QSocketEventEmetterBase {
  /**
   * Main implementation of the `on` method, determining which handler to add.
   * @param {string} event - Event name.
   * @param {Function} listener - Callback function for the event.
   * @param {TQSocketContentType} [contentType] - Optional content type.
   * @param {TQSocketContentEncoding} [contentEncoding] - Optional encoding: 'raw' | 'gzip' | 'deflate'.
   */
  on(event, listener, contentType, contentEncoding) {
    if (event === "disconnection") {
      this.disconnectionListeners.push(listener);
    } else {
      this.addEventListener(event, listener, 0 /* ON */, contentType, contentEncoding);
    }
  }
  /**
   * Main implementation of the `once` method, determining the addition of a one-time handler.
   * @param {string} event - Event name.
   * @param {Function} listener - Callback function for the event.
   * @param {TQSocketContentType} [contentType] - Optional content type.
   * @param {TQSocketContentEncoding} [contentEncoding] - Optional encoding: 'raw' | 'gzip' | 'deflate'.
   */
  once(event, listener, contentType, contentEncoding) {
    if (event === "disconnection") {
      this.disconnectionListeners.push(listener);
    } else {
      this.addEventListener(event, listener, 0 /* ON */, contentType, contentEncoding);
    }
  }
  /**
   * Removes a listener for a custom event.
   * @example
   * ```typescript
   * emitter.off('customEvent', customEventHandler);
   * ```
   * @param {string} event - Custom event name.
   * @param {Function} listener - Callback function registered for the event.
   */
  off(event, listener) {
    if (event === "disconnection") {
      const index = this.disconnectionListeners.lastIndexOf(listener);
      if (index !== -1) this.disconnectionListeners.splice(index, 1);
    } else {
      this.removeEventListener(event, listener);
    }
  }
};
var QSocketNamespaceEventEmitter = class extends QSocketEventEmetterBase {
  /**
   * Main implementation of the `on` method, determining which handler to add.
   * @param {string} event - Event name.
   * @param {Function} listener - Callback function for the event.
   * @param {TQSocketContentType} [contentType] - Optional content type.
   * @param {TQSocketContentEncoding} [contentEncoding] - Optional encoding: 'raw' | 'gzip' | 'deflate'.
   */
  on(event, listener, contentType, contentEncoding) {
    if (event === "connection") {
      this.connectionListeners.push(listener);
    } else if (event === "disconnection") {
      this.disconnectionListeners.push(listener);
    } else {
      this.addEventListener(event, listener, 0 /* ON */, contentType, contentEncoding);
    }
  }
  /**
   * Main implementation of the `once` method, determining the addition of a one-time handler.
   * @param {string} event - Event name.
   * @param {Function} listener - Callback function for the event.
   * @param {TQSocketContentType} [contentType] - Optional content type.
   * @param {TQSocketContentEncoding} [contentEncoding] - Optional encoding: 'raw' | 'gzip' | 'deflate'.
   */
  once(event, listener, contentType, contentEncoding) {
    if (event === "connection") {
      this.connectionListeners.push(listener);
    } else if (event === "disconnection") {
      this.disconnectionListeners.push(listener);
    } else {
      this.addEventListener(event, listener, 0 /* ON */, contentType, contentEncoding);
    }
  }
  /**
   * Removes a listener for a custom event.
   * @example
   * ```typescript
   * emitter.off('customEvent', customEventHandler);
   * ```
   * @param {string} event - Custom event name.
   * @param {Function} listener - Callback function registered for the event.
   */
  off(event, listener) {
    if (event === "connection") {
      const index = this.connectionListeners.lastIndexOf(listener);
      if (index !== -1) this.connectionListeners.splice(index, 1);
    } else if (event === "disconnection") {
      const index = this.disconnectionListeners.lastIndexOf(listener);
      if (index !== -1) this.disconnectionListeners.splice(index, 1);
    } else {
      this.removeEventListener(event, listener);
    }
  }
};

// src/core/QSocketConnection.ts
var QSocketConnection = class extends QSocketConnectionEventEmitter {
  //#endregion
  //#region Конструктор
  constructor(interaction, namespace) {
    super();
    this.interaction = interaction;
    this.namespace = namespace;
  }
  //#endregion
  //#region Методы управления соединением
  static close(connection) {
    connection.close();
  }
  close() {
    this.disconnectionListeners.forEach((listener) => listener());
    this.listeners.clear();
  }
  //#endregion
  //#region Методы отправки и передачи данных
  /**
   * @description Отправка данных на связанный клиент
   */
  async emit(event, data, contentType, contentEncoding) {
    const message = [
      {
        payload: {
          data,
          "Content-Type": determineContentType(data, contentType),
          "Content-Encoding": determineContentEncoding(contentEncoding)
        },
        meta: {
          type: EQSocketProtocolMessageType.DATA,
          uuid: this.interaction.uuid.next(),
          namespace: this.namespace.name,
          event
        }
      }
    ];
    console.log("\u041F\u041E\u041B\u0415\u0422\u0415\u041B\u041E \u0421\u041E\u041E\u0411\u0429\u0415\u041D\u0418\u0426\u0415", message);
    const returns = await this.interaction.sendData(message);
    return returns === void 0 ? [] : returns[0];
  }
  async broadcast(event, data, contentType, contentEncoding) {
    const message = {
      payload: {
        data,
        "Content-Type": determineContentType(data, contentType),
        "Content-Encoding": determineContentEncoding(contentEncoding)
      },
      meta: {
        type: EQSocketProtocolMessageType.DATA,
        uuid: this.interaction.uuid.next(),
        namespace: this.namespace.name,
        event
      }
    };
    const interactionsResults = await this.interaction.broadcast([message]);
    return interactionsResults.map((interactionResult) => interactionResult[0]);
  }
  static async pipe(connection, message) {
    return await connection.$__pipe(message);
  }
  /**
   * @description Передача сообщения из пространства имён в соединение
   */
  async $__pipe(chunk) {
    const event = chunk.meta.event;
    const payload = chunk.payload;
    const events = this.listeners.get(event);
    if (!events) return [];
    this.listeners.set(
      event,
      events.filter((event2) => event2.type === 0 /* ON */)
    );
    const results = await Promise.allSettled(
      events.map(async (eventInstance) => {
        const data = await eventInstance.listener(
          payload.data,
          getContentTypeString(payload["Content-Type"]),
          getContentEncodingString(payload["Content-Encoding"])
        );
        return {
          payload: {
            data,
            "Content-Type": determineContentType(data, eventInstance.contentType),
            "Content-Encoding": determineContentEncoding(eventInstance.contentEncoding)
          },
          meta: {
            type: EQSocketProtocolMessageType.ACK,
            uuid: chunk.meta.uuid
          }
        };
      })
    );
    return results.reduce((acc, cur) => {
      if (cur.status === "fulfilled" && cur.value) acc.push(cur.value);
      return acc;
    }, []);
  }
  //#endregion
};

// src/core/QSocketNamespace.ts
var QSocketNamespace = class extends QSocketNamespaceEventEmitter {
  constructor(name, debuger) {
    super();
    this.connections = /* @__PURE__ */ new Map();
    this.middlewares = [];
    this._name = name;
    this.debuger = debuger;
  }
  get name() {
    return this._name;
  }
  use(handler) {
    this.middlewares.push(handler);
  }
  close() {
    this.connections.forEach((_, interaction) => {
      this.$__deleteClient(interaction);
    });
  }
  //#region Методы событий
  async emit(event, data, contentType, contentEncoding) {
    const promises = [];
    console.log("\u041F\u042B\u0422\u0410\u0415\u0422\u0421\u042F \u0412 \u041D\u042D\u0419\u041C\u0421\u041F\u0415\u0419\u0421\u041D\u042B\u0419 \u042D\u041C\u0418\u0422", this.connections, data);
    this.connections.forEach((connection) => {
      promises.push(connection.emit(event, data, contentType, contentEncoding));
    });
    return (await Promise.allSettled(promises)).filter((res) => res.status === "fulfilled").map(({ value }) => value);
  }
  //#endregion
  //#region Методы управления потоком данных
  static async pipe(interaction, namespace, message) {
    return await namespace.$__pipe(interaction, message);
  }
  async $__pipe(interaction, chunk) {
    const connection = this.connections.get(interaction);
    if (!connection) {
      return [];
    }
    let errors = [];
    this.middlewares.forEach((middleware) => {
      try {
        chunk = middleware(chunk);
      } catch (error) {
        if (error instanceof Error) errors.push(error);
        else errors.push(new Error("Unknown error"));
      }
    });
    if (errors.length > 0) return [createErrorAckMessage(chunk, errors)];
    return await QSocketConnection.pipe(connection, chunk);
  }
  //#endregion
  //#region Методы управления клиентами
  static addClient(namespace, interaction) {
    namespace.$__addClient(interaction);
  }
  $__addClient(interaction) {
    const connection = new QSocketConnection(interaction, this);
    this.connections.set(interaction, connection);
    this.connectionListeners.forEach((listener) => {
      try {
        listener(connection);
      } catch (error) {
        this.debuger.error("Connection event error:", error);
      }
    });
    this.debuger.info(`Interaction "${interaction.id}" join namespace "${this.name}"`);
  }
  static deleteClient(namespace, interaction) {
    namespace.$__deleteClient(interaction);
  }
  $__deleteClient(interaction) {
    const connection = this.connections.get(interaction);
    this.connections.delete(interaction);
    this.disconnectionListeners.forEach((listener) => {
      try {
        listener();
      } catch (e) {
      }
    });
    this.debuger.info(`Interaction "${interaction.id}" leave namespace "${this.name}"`);
    if (connection !== void 0) {
      QSocketConnection.close(connection);
    }
  }
  //#endregion
};

// src/core/QSocketDebuger.ts
var QSocketDebuger = class {
  /**
   * Конструктор класса QSocketUtils.
   * @param {IQSocketConfigBase['debug']} [debugConfig] - Конфигурация для режима отладки.
   */
  constructor(debugConfig) {
    /** Логгер, используемый для вывода сообщений */
    this.logger = console;
    const { enabled = false, logger = console, prefix = "" } = debugConfig != null ? debugConfig : {};
    this.enabled = enabled;
    this.logger = logger;
    this.prefix = prefix;
  }
  /**
   * Логирует сообщение, если включен режим отладки.
   * @param {...any[]} message - Сообщение или данные для логирования.
   */
  log(...message) {
    if (this.enabled) this.logger.log(this.prefix, ...message);
  }
  /**
   * Логирует сообщение об ошибке, если включен режим отладки.
   * @param {...any[]} message - Сообщение или данные для логирования ошибок.
   */
  error(...message) {
    if (this.enabled) this.logger.error(this.prefix, ...message);
  }
  /**
   * Логирует информационное сообщение, если включен режим отладки.
   * @param {...any[]} message - Сообщение или данные для информационного логирования.
   */
  info(...message) {
    if (this.enabled) this.logger.info(this.prefix, ...message);
  }
  /**
   * Логирует предупреждение, если включен режим отладки.
   * @param {...any[]} message - Сообщение или данные для логирования предупреждений.
   */
  warn(...message) {
    if (this.enabled) this.logger.warn(this.prefix, ...message);
  }
  //#endregion
};

// src/core/QSocketUniqueGenerator.ts
var MAX_VALUE = Number.MAX_SAFE_INTEGER;
var QSocketUniqueGenerator = class {
  constructor(prefix = "") {
    /**
     * Текущий индекс для генерации UUID.
     * @private
     */
    this.uuidIndex = 0;
    this.prefix = prefix;
  }
  /**
   * Метод для генерации следующего уникального идентификатора.
   */
  next() {
    if (++this.uuidIndex > MAX_VALUE) this.uuidIndex = 0;
    return `${this.prefix}${this.uuidIndex.toString(16)}`;
  }
};

// src/core/QSocketInteraction.ts
var QSocketInteraction = class {
  constructor(id, socket, allNamespaces = /* @__PURE__ */ new Map(), interactions, middlewares = [], protocol, debuger) {
    this.acks = /* @__PURE__ */ new Map();
    this.connectedNamespaces = /* @__PURE__ */ new Map();
    this.allNamespaces = /* @__PURE__ */ new Map();
    this.middlewares = [];
    this.id = id;
    this.uuid = new QSocketUniqueGenerator(`${this.id}-M`);
    this.debuger = debuger;
    this.socket = socket;
    this.interactions = interactions;
    this.allNamespaces = allNamespaces;
    this.middlewares = middlewares;
    this.protocol = protocol;
    this.socket.on("message", this.onHandle.bind(this));
  }
  static close(interaction) {
    interaction.socket.close();
    interaction.closeHandle();
  }
  closeHandle() {
    this.debuger.log("\u0417\u0430\u043F\u0443\u0449\u0435\u043D \u043F\u0440\u043E\u0446\u0435\u0441\u0441 \u0443\u043D\u0438\u0447\u0442\u043E\u0436\u0435\u043D\u0438\u044F \u0441\u043E\u0435\u0434\u0438\u043D\u0435\u043D\u0438\u044F", this.id);
    this.acks.clear();
    this.socket.close();
    this.connectedNamespaces.forEach((namespace) => QSocketNamespace.deleteClient(namespace, this));
  }
  //#region ПРОСЛУШИВАНИЕ СОБЫТИЙ
  async onHandle(data) {
    if (typeof data === "string") {
      this.debuger.error("\u041E\u0431\u0449\u0435\u043D\u0438\u0435 \u043F\u043E \u043F\u0440\u043E\u0442\u043E\u043A\u043E\u043B\u0443 QSocket \u0432\u043E\u0437\u043C\u043E\u0436\u043D\u043E \u0442\u043E\u043B\u044C\u043A\u043E \u0432 \u0444\u043E\u0440\u043C\u0430\u0442\u0435 \u0431\u0443\u0444\u0435\u0440\u0430");
      return;
    }
    const buffer = new Uint8Array(data);
    let message = await this.protocol.from(buffer);
    if (message instanceof Error) {
      this.debuger.error(message);
      return;
    }
    const ackChunks = [];
    const dataChunks = [];
    const controlChunks = [];
    message.forEach((chunk) => {
      switch (chunk.meta.type) {
        case EQSocketProtocolMessageType.DATA:
          dataChunks.push(chunk);
          break;
        case EQSocketProtocolMessageType.ACK:
          ackChunks.push(chunk);
          break;
        case EQSocketProtocolMessageType.CONTROL:
          controlChunks.push(chunk);
          break;
      }
    });
    if (dataChunks.length > 0) {
      const errors = [];
      let middlewareResult = dataChunks;
      this.middlewares.forEach((middleware) => {
        middlewareResult = middleware(middlewareResult, this.socket);
        if (middlewareResult instanceof Error) {
          errors.push(middlewareResult);
          return;
        }
      });
      if (errors.length > 0) {
        this.sendAck(middlewareResult.map((chunk) => createErrorAckMessage(chunk, errors)));
      } else this.onData(middlewareResult);
    }
    if (ackChunks.length > 0) {
      this.onAck(ackChunks);
    }
    if (controlChunks.length > 0) {
      this.onControl(controlChunks);
    }
  }
  onData(message) {
    message.forEach((chunk) => {
      const namespaceInstance = this.connectedNamespaces.get(chunk.meta.namespace);
      if (!namespaceInstance) {
        this.debuger.error(`Namespace "${chunk.meta.namespace}" does not exist`);
        return;
      }
      QSocketNamespace.pipe(this, namespaceInstance, chunk).then((results) => {
        this.sendAck(results);
      }).catch((error) => {
        this.debuger.error(`\u041E\u0448\u0438\u0431\u043A\u0430 \u043F\u0440\u0438 \u043E\u0431\u0440\u0430\u0431\u043E\u0442\u043A\u0435 \u0434\u0430\u043D\u043D\u044B\u0445: ${error.message}`);
        this.sendAck([
          {
            meta: {
              type: EQSocketProtocolMessageType.ACK,
              uuid: chunk.meta.uuid
            },
            payload: {
              data: `Error: ${error.message}`,
              "Content-Type": EQSocketProtocolContentType.STRING,
              "Content-Encoding": EQSocketProtocolContentEncoding.RAW
            }
          }
        ]);
      });
    });
  }
  onAck(message) {
    const ackMap = /* @__PURE__ */ new Map();
    message.forEach((chunk) => {
      let ack = ackMap.get(chunk.meta.uuid) || [];
      ack.push(chunk);
      ackMap.set(chunk.meta.uuid, ack);
    });
    ackMap.forEach((value, uuid) => {
      const resolve = this.acks.get(uuid);
      if (!resolve) {
        this.debuger.error(`Return message UUID not found [id: ${this.id}, uuid: ${uuid}]`);
        return;
      }
      resolve(value.map((item) => item.payload));
      this.acks.delete(uuid);
    });
  }
  onControl(message) {
    message.forEach(async (chunk) => {
      const data = chunk.payload.data;
      if (data.command === "ping") {
        await this.sendAck(createPongMessage(chunk.meta.uuid));
      } else if (data.command === "join-namespace") {
        const namespace = this.allNamespaces.get(data.namespace);
        if (!namespace) {
          this.debuger.error(`Namespace "${data.namespace}" not found`);
          return;
        }
        await this.sendAck(createConfirmAckMessage(chunk, true));
        if (!this.connectedNamespaces.has(namespace.name)) {
          this.connectedNamespaces.set(namespace.name, namespace);
        }
        QSocketNamespace.addClient(namespace, this);
      } else if (data.command === "leave-from-namespace") {
        if (typeof data.namespace === "string") {
          const namespace = this.connectedNamespaces.get(data.namespace);
          if (!namespace) {
            this.debuger.error(`Namespace "${data.namespace}" not found`);
            return;
          }
          await this.sendAck(createConfirmAckMessage(chunk, true));
          this.connectedNamespaces.delete(namespace.name);
          QSocketNamespace.deleteClient(namespace, this);
        }
      } else {
        this.debuger.error("Unknown control command");
      }
    });
  }
  //#endregion
  //#region ОТПРАВКА ДАННЫХ
  async broadcast(message) {
    const promises = [];
    this.interactions.forEach((interaction) => {
      if (interaction !== this) {
        promises.push(interaction.sendData(message));
      }
    });
    const interactionsResults = await Promise.allSettled(promises).then(
      (result) => result.filter((item) => item.status === "fulfilled").map(({ value }) => value).filter((value) => value !== void 0)
    );
    return interactionsResults;
  }
  async sendData(message) {
    const data = await this.protocol.to(message);
    if (data instanceof Error) {
      this.debuger.error(data);
      return;
    }
    this.socket.send(data);
    const result = (await Promise.allSettled(
      message.map((chunk) => {
        return new Promise((emitResolve, emitReject) => {
          const timeout = 1e4;
          const ackResolver = (ackResult) => {
            clearTimeout(timer);
            this.acks.delete(chunk.meta.uuid);
            emitResolve(ackResult);
          };
          this.acks.set(chunk.meta.uuid, ackResolver);
          const timer = setTimeout(() => {
            this.acks.delete(chunk.meta.uuid);
            this.debuger.error(`\u0412\u0440\u0435\u043C\u044F \u043E\u0436\u0438\u0434\u0430\u043D\u0438\u044F \u0438\u0441\u0442\u0435\u043A\u043B\u043E [event: ${chunk.meta.event}, uuid: ${chunk.meta.uuid}]`);
            emitReject(new Error("Timeout"));
          }, timeout);
        });
      })
    )).filter((res) => res.status === "fulfilled").map(({ value }) => value);
    return result;
  }
  async sendAck(message) {
    const data = await this.protocol.to(message);
    this.socket.send(data);
  }
  async sendCommand(message) {
    const data = await this.protocol.to(message);
    if (data instanceof Error) {
      this.debuger.error(data);
      return;
    }
    this.socket.send(data);
    return (await Promise.allSettled(
      message.map((chunk) => {
        return new Promise((emitResolve, emitReject) => {
          const timeout = 1e4;
          const ackResolver = (ackResult) => {
            clearTimeout(timer);
            emitResolve(ackResult);
            this.acks.delete(chunk.meta.uuid);
          };
          this.acks.set(chunk.meta.uuid, ackResolver);
          const timer = setTimeout(() => {
            this.debuger.error(`\u0412\u0440\u0435\u043C\u044F \u043E\u0436\u0438\u0434\u0430\u043D\u0438\u044F \u0438\u0441\u0442\u0435\u043A\u043B\u043E [command, uuid: ${chunk.meta.uuid}]`);
            emitReject(new Error("Timeout"));
            this.acks.delete(chunk.meta.uuid);
          }, timeout);
        });
      })
    )).filter((res) => res.status === "fulfilled").map(({ value }) => value);
  }
  //#endregion
  //#region NAMESPACES
  static joinNamespace(interaction, namespace) {
    interaction.joinNamespace(namespace);
  }
  joinNamespace(namespace) {
    this.connectedNamespaces.set(namespace.name, namespace);
    QSocketNamespace.addClient(namespace, this);
  }
  static leaveNamespace(interaction, namespace) {
    interaction.leaveNamespace(namespace);
  }
  leaveNamespace(namespace) {
    this.connectedNamespaces.delete(namespace.name);
    QSocketNamespace.deleteClient(namespace, this);
  }
  //#endregion
};

// src/interfaces/QSocketBase.ts
var clientUUID = new QSocketUniqueGenerator();
var serverUUID = new QSocketUniqueGenerator();
var QSocketBase = class {
  constructor(type, config) {
    this.namespaces = /* @__PURE__ */ new Map();
    this.interactions = /* @__PURE__ */ new Map();
    this.middlewares = [];
    var _a, _b, _c;
    this.type = type;
    this.debuger = new QSocketDebuger(config == null ? void 0 : config.debug);
    this.protocol = new QSocketProtocol((_a = config == null ? void 0 : config.compression) == null ? void 0 : _a.compressor, (_c = (_b = config == null ? void 0 : config.compression) == null ? void 0 : _b.compressionFromSize) != null ? _c : 1024 * 100);
    const prefix = type === "server" ? "S" : "C";
    const generator = type === "server" ? serverUUID : clientUUID;
    this.id = `${prefix}${generator.next()}`;
    this.uuid = new QSocketUniqueGenerator(`${this.id}-SM`);
    this.interactionUUID = new QSocketUniqueGenerator(`${this.id}-I`);
  }
  connectionHandle(socket) {
    const interactionId = this.interactionUUID.next();
    const interaction = new QSocketInteraction(interactionId, socket, this.namespaces, this.interactions, this.middlewares, this.protocol, this.debuger);
    this.interactions.set(interactionId, interaction);
    socket.on("close", () => this.closeInteraction(interactionId, interaction));
  }
  closeInteraction(interactionId, interaction) {
    QSocketInteraction.close(interaction);
    this.interactions.delete(interactionId);
  }
  /**
   * Создаёт новое пространство имён или возвращает существующее.
   * @param {string} name - Имя создаваемого пространства имён.
   * @returns {QSocketNamespace} Пространство имён QSocket.
   */
  createNamespace(name) {
    if (this.namespaces.has(name)) {
      this.debuger.warn(`The namespace "${name}" already exists.`);
      return this.namespaces.get(name);
    }
    const namespace = new QSocketNamespace(name, this.debuger);
    this.namespaces.set(name, namespace);
    this.debuger.info(`The namespace "${name}" has been created.`);
    this.afterCreatingNamespace(namespace);
    return namespace;
  }
  /**
   * Удаляет существующее пространство имён.
   * @param {string} name - Имя удаляемого пространства имён.
   * @returns {boolean} Возвращает `true`, если пространство имён было удалено, иначе `false`.
   */
  deleteNamespace(name) {
    const namespace = this.namespaces.get(name);
    if (namespace === void 0) {
      this.debuger.warn(`The namespace '${name}' does not exist.`);
      return false;
    }
    this.namespaces.delete(name);
    namespace.close();
    this.interactions.forEach((interaction) => QSocketInteraction.leaveNamespace(interaction, namespace));
    this.debuger.info(`The namespace "${name}" has been removed.`);
    return true;
  }
  /**
   * @description Добавляет промежуточный обработчик сообщений
   * @param handler
   */
  use(handler) {
    this.middlewares.push(handler);
  }
};

// src/interfaces/QSocketClient.ts
var QSocketClient = class extends QSocketBase {
  constructor(socketBuilder, config) {
    super("client", config);
    this.isConnected = false;
    this.isIntentionalDisconnect = false;
    this.reconnectionAttempts = 0;
    this.reconnectionConfig = config == null ? void 0 : config.reconnection;
    this.transportBuilder = socketBuilder;
  }
  connect() {
    if (this.isConnected) this.disconnect();
    this.isIntentionalDisconnect = false;
    const transport = this.transportBuilder();
    transport.on("close", () => {
      this.isConnected = false;
      if (!this.isIntentionalDisconnect) this.startReconnection();
    });
    this.connectionHandle(transport);
    this.isConnected = true;
    this.namespaces.forEach(this.afterCreatingNamespace.bind(this));
    return this;
  }
  disconnect() {
    this.isIntentionalDisconnect = true;
    this.interactions.forEach(QSocketInteraction.close);
    this.isConnected = false;
  }
  reconnect() {
    this.disconnect();
    this.connect();
  }
  //#region Абстрактные методы
  afterCreatingNamespace(namespace) {
    const message = [
      {
        meta: {
          type: EQSocketProtocolMessageType.CONTROL,
          uuid: this.uuid.next()
        },
        payload: {
          data: {
            command: "join-namespace",
            namespace: namespace.name
          },
          "Content-Type": EQSocketProtocolContentType.JSON,
          "Content-Encoding": EQSocketProtocolContentEncoding.RAW
        }
      }
    ];
    this.interactions.forEach((interaction) => {
      interaction.sendCommand(message).then(() => QSocketInteraction.joinNamespace(interaction, namespace)).catch(() => this.debuger.error(`\u041E\u0448\u0438\u0431\u043A\u0430 \u043F\u0440\u0438 \u043F\u043E\u0434\u043A\u043B\u044E\u0447\u0435\u043D\u0438\u0438 \u043A \u043F\u0440\u043E\u0441\u0442\u0440\u0430\u043D\u0441\u0442\u0432\u0443 "${namespace.name}"`));
    });
  }
  //#endregion
  //#region
  startReconnection() {
    const { enabled, maxAttempts = 5, delay = 1e3 } = this.reconnectionConfig || {};
    if (!enabled) return;
    const attemptReconnection = () => {
      if (this.reconnectionAttempts >= maxAttempts) {
        this.debuger.error("Exceeded maximum reconnection attempts");
        return;
      }
      this.reconnectionAttempts++;
      this.connect();
      const checkConnection = () => {
        if (this.isConnected) {
          this.sendPingMessage().then(() => {
            this.debuger.info("Reconnection successful");
            this.reconnectionAttempts = 0;
          }).catch(() => {
            this.scheduleNextReconnection();
          });
        } else {
          this.scheduleNextReconnection();
        }
      };
      setTimeout(checkConnection, delay);
    };
    attemptReconnection();
  }
  // Метод для планирования следующей попытки переподключения
  scheduleNextReconnection() {
    const { delay = 1e3, exponentialBackoff = false } = this.reconnectionConfig || {};
    const nextDelay = exponentialBackoff ? delay * 2 ** this.reconnectionAttempts : delay;
    setTimeout(() => this.startReconnection(), nextDelay);
  }
  // Метод для отправки "ping" сообщения для проверки состояния подключения
  async sendPingMessage() {
    const pingPromises = Array.from(this.interactions.values()).map((interaction) => interaction.sendCommand(createPingMessage(this.uuid.next())));
    try {
      await Promise.all(pingPromises);
      return await Promise.resolve();
    } catch (e) {
      return await Promise.reject();
    }
  }
  //#endregion
};

// src/interfaces/QSocketServer.ts
var QSocketServer = class extends QSocketBase {
  constructor(transport, config) {
    super("server", config);
    this.server = transport;
    this.server.on("connection", (socket) => this.connectionHandle(socket));
  }
  //#region Абстрактные методы
  afterCreatingNamespace() {
  }
  //#endregion
};

export { QSocketClient, QSocketServer };
//# sourceMappingURL=index.js.map
//# sourceMappingURL=index.js.map